from http import HTTPStatus
from typing import Any, Dict, Optional, Union

import httpx

from ... import errors
from ...client import Client
from ...models.generate_names_namer_get_response_generate_names_namer_get import (
    GenerateNamesNamerGetResponseGenerateNamesNamerGet,
)
from ...models.http_validation_error import HTTPValidationError
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    client: Client,
    phrase: str,
    gh_token: Union[Unset, str] = UNSET,
) -> Dict[str, Any]:
    url = "{}/namer".format(client.base_url)

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    if not isinstance(gh_token, Unset):
        headers["gh-token"] = gh_token

    params: Dict[str, Any] = {}
    params["phrase"] = phrase

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "method": "get",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "follow_redirects": client.follow_redirects,
        "params": params,
    }


def _parse_response(
    *, client: Client, response: httpx.Response
) -> Optional[Union[GenerateNamesNamerGetResponseGenerateNamesNamerGet, HTTPValidationError]]:
    if response.status_code == HTTPStatus.OK:
        response_200 = GenerateNamesNamerGetResponseGenerateNamesNamerGet.from_dict(response.json())

        return response_200
    if response.status_code == HTTPStatus.UNPROCESSABLE_ENTITY:
        response_422 = HTTPValidationError.from_dict(response.json())

        return response_422
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Client, response: httpx.Response
) -> Response[Union[GenerateNamesNamerGetResponseGenerateNamesNamerGet, HTTPValidationError]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: Client,
    phrase: str,
    gh_token: Union[Unset, str] = UNSET,
) -> Response[Union[GenerateNamesNamerGetResponseGenerateNamesNamerGet, HTTPValidationError]]:
    """Generate Names

    Args:
        phrase (str):
        gh_token (Union[Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[GenerateNamesNamerGetResponseGenerateNamesNamerGet, HTTPValidationError]]
    """

    kwargs = _get_kwargs(
        client=client,
        phrase=phrase,
        gh_token=gh_token,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: Client,
    phrase: str,
    gh_token: Union[Unset, str] = UNSET,
) -> Optional[Union[GenerateNamesNamerGetResponseGenerateNamesNamerGet, HTTPValidationError]]:
    """Generate Names

    Args:
        phrase (str):
        gh_token (Union[Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[GenerateNamesNamerGetResponseGenerateNamesNamerGet, HTTPValidationError]
    """

    return sync_detailed(
        client=client,
        phrase=phrase,
        gh_token=gh_token,
    ).parsed


async def asyncio_detailed(
    *,
    client: Client,
    phrase: str,
    gh_token: Union[Unset, str] = UNSET,
) -> Response[Union[GenerateNamesNamerGetResponseGenerateNamesNamerGet, HTTPValidationError]]:
    """Generate Names

    Args:
        phrase (str):
        gh_token (Union[Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[GenerateNamesNamerGetResponseGenerateNamesNamerGet, HTTPValidationError]]
    """

    kwargs = _get_kwargs(
        client=client,
        phrase=phrase,
        gh_token=gh_token,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: Client,
    phrase: str,
    gh_token: Union[Unset, str] = UNSET,
) -> Optional[Union[GenerateNamesNamerGetResponseGenerateNamesNamerGet, HTTPValidationError]]:
    """Generate Names

    Args:
        phrase (str):
        gh_token (Union[Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[GenerateNamesNamerGetResponseGenerateNamesNamerGet, HTTPValidationError]
    """

    return (
        await asyncio_detailed(
            client=client,
            phrase=phrase,
            gh_token=gh_token,
        )
    ).parsed
